@@ -3,7 +3,7 @@
 
  Lincoln Stein <lincoln.stein@gmail.com>
  Ben Faga <ben.faga@gmail.com>
- $Id: controller.js 22998 2010-04-05 18:16:38Z rbuels $
+ $Id: controller.js 23191 2010-05-13 03:20:48Z lstein $
 
 Indentation courtesy of Emacs javascript-mode 
 (http://mihai.bazon.net/projects/emacs-javascript-mode/javascript.el)
@@ -52,6 +52,30 @@ var GBrowseController = Class.create({
     this.segment_info;
     this.last_update_key;
     this.tabs;
+
+    //global config variables
+    this.globals = new Hash();
+
+  },
+
+  set_globals:
+  function(obj) {
+    for(var name in obj) {
+      this.globals.set(name, obj[name])
+    }
+
+    var me = this;
+
+    //generate *_url accessors
+    var mk_url_accessor = function( conf_name, acc_name) {
+      me[acc_name] = function(relpath) { return this.globals.get(conf_name) + '/' + relpath; }
+    };
+    mk_url_accessor( 'buttons',      'button_url'     );
+    mk_url_accessor( 'balloons',     'balloon_url'    );
+    mk_url_accessor( 'openid',       'openid_url'     );
+    mk_url_accessor( 'js',           'js_url'         );
+    mk_url_accessor( 'gbrowse_help', 'help_url'       );
+    mk_url_accessor( 'stylesheet',   'stylesheet_url' );
   },
 
   reset_after_track_load:
@@ -241,7 +265,7 @@ var GBrowseController = Class.create({
     var request_str = "action=update_sections" + param_str;
     for (var i = 0; i < section_names.length; i++) {
       if (spin)
-         $(section_names[i]).innerHTML="<img src='/gbrowse2/images/spinner.gif' alt='loading...' />";
+        $(section_names[i]).innerHTML = '<img src="' + this.button_url('spinner.gif') + '" alt="Working..." />';
       request_str += "&section_names="+section_names[i];
     }
 
@@ -368,7 +392,7 @@ var GBrowseController = Class.create({
         if (results.display_details == 0){
           Controller.hide_detail_tracks();
         }
-	Controller.idle();
+//	Controller.idle();
       } // end onSuccess
       
     }); // end Ajax.Request
@@ -404,6 +428,7 @@ var GBrowseController = Class.create({
 
     if (!found_track) return false;
 
+    this.busy();
     new Ajax.Request(document.URL,{
       method:     'post',
       parameters: request_str,
@@ -456,8 +481,8 @@ var GBrowseController = Class.create({
   function() {
     var bi = $('busy_indicator');
     var top  = document.body.scrollTop||document.documentElement.scrollTop;
-    bi.style.top=top+"px";
-    bi.style.left=5+"px";
+    bi.style.top  =top+5+"px";
+    bi.style.left =5+"px";
     bi.show();
   },
 
@@ -558,6 +583,7 @@ var GBrowseController = Class.create({
     var track_ids = [];
     var finished = true;
     var track_key_str = '';
+    this.busy();
     this.retrieve_tracks.keys().each(
       function(track_id) {
         if(Controller.retrieve_tracks.get(track_id)){
@@ -571,7 +597,7 @@ var GBrowseController = Class.create({
     );
 
     if (finished) {
-      this.idle();
+//      this.idle();
       return;
     }
 
@@ -846,7 +872,7 @@ var GBrowseController = Class.create({
       if (event.type=='blur' || event.keyCode==Event.KEY_RETURN) {
 	  var upload_name = el.id.sub('_description$','');
 	  var desc        = el.innerHTML;
-	  el.innerHTML  = "<img src='/gbrowse2/images/spinner.gif' alt='loading...' />";
+	  el.innerHTML  = '<img src="' + this.button_url('spinner.gif') + ' alt="Working..." />';
 	  new Ajax.Request(document.URL, {
 		      method:      'post',
 		      parameters:{  
@@ -864,7 +890,7 @@ var GBrowseController = Class.create({
 	  return true;
       }
       if (event.keyCode==Event.KEY_ESC) {
-	  el.innerHTML  = "<img src='/gbrowse2/images/spinner.gif' alt='loading...' />";
+          el.innerHTML  = '<img src="' + this.button_url('spinner.gif') + '" alt="Working..." />';
 	  Controller.update_sections(new Array(userdata_table_id,userimport_table_id));
 	  el.stopObserving('keypress');
 	  el.stopObserving('blur');
